@@ -3,12 +3,12 @@
  *
  * This file is part of msmtp, an SMTP client.
  *
- * Copyright (C) 2000, 2003, 2004, 2005, 2006
+ * Copyright (C) 2000, 2003, 2004, 2005, 2006, 2007
  * Martin Lambers <marlam@marlam.de>
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   the Free Software Foundation; either version 3 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
@@ -17,12 +17,11 @@
  *   GNU General Public License for more details.
  *
  *   You should have received a copy of the GNU General Public License
- *   along with this program; if not, write to the Free Software Foundation,
- *   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #ifdef HAVE_CONFIG_H
-#include "config.h"
+# include "config.h"
 #endif
 
 #include <stdio.h>
@@ -31,19 +30,17 @@
 #include <string.h>
 #include <ctype.h>
 #include <errno.h>
-extern int errno;
 #include <time.h>
 #include <getopt.h>
 extern char *optarg;
 extern int optind;
+#include <unistd.h>
+#include <sysexits.h>
 #ifdef ENABLE_NLS
-#include <locale.h>
+# include <locale.h>
 #endif
 #ifdef HAVE_SYSLOG
-#include <syslog.h>
-#endif
-#ifdef HAVE_UNISTD_H
-#include <unistd.h>
+# include <syslog.h>
 #endif
 #ifdef W32_NATIVE
 #include <io.h>
@@ -52,28 +49,28 @@ extern int optind;
 #include <winsock2.h>
 #elif defined DJGPP
 #include <io.h>
+#include <fcntl.h>
 #include <netdb.h>
 #include <arpa/inet.h>
 #else /* UNIX */
 #include <netdb.h>
 #include <arpa/inet.h>
 #endif
-#include <sysexits.h>
 
 #include "getpass.h"
 #include "gettext.h"
 #include "xalloc.h"
 #include "xvasprintf.h"
 
-#include "list.h"
-#include "os_env.h"
 #include "conf.h"
+#include "list.h"
 #include "net.h"
 #include "netrc.h"
 #include "smtp.h"
-#ifdef HAVE_SSL
-#include "tls.h"
-#endif /* HAVE_SSL */
+#include "tools.h"
+#ifdef HAVE_TLS
+# include "tls.h"
+#endif /* HAVE_TLS */
 
 /* Default file names. */
 #ifdef W32_NATIVE
@@ -132,7 +129,7 @@ int exitcode_net(int net_error_code)
     }
 }
 
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 int exitcode_tls(int tls_error_code)
 {
     switch (tls_error_code)
@@ -156,7 +153,7 @@ int exitcode_tls(int tls_error_code)
 	    return EX_SOFTWARE;
     }
 }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
 int exitcode_smtp(int smtp_error_code)
 {
@@ -322,7 +319,7 @@ char *msmtp_password_callback(const char *hostname, const char *user)
  * Prints information about a TLS certificate.
  */
 
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 /* Convert the given time into a string. */
 void msmtp_time_to_string(time_t *t, char *buf, size_t bufsize)
 {
@@ -440,9 +437,9 @@ int msmtp_rmqs(account_t *acc, int debug, const char *rmqs_argument,
 {
     smtp_server_t srv;
     int e;
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     tls_cert_info_t *tci = NULL;
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
     
     *errstr = NULL;
     *msg = NULL;
@@ -458,19 +455,20 @@ int msmtp_rmqs(account_t *acc, int debug, const char *rmqs_argument,
     }
 
     /* prepare tls */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls)
     {
 	if ((e = smtp_tls_init(&srv, acc->tls_key_file, acc->tls_cert_file, 
-			acc->tls_trust_file, errstr)) != TLS_EOK)
+			acc->tls_trust_file, acc->tls_force_sslv3, errstr)) 
+		!= TLS_EOK)
 	{
 	    return exitcode_tls(e);
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     /* start tls for ssmtp servers */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls && acc->tls_nostarttls)
     {
 	if (debug)
@@ -493,7 +491,7 @@ int msmtp_rmqs(account_t *acc, int debug, const char *rmqs_argument,
 	    tls_cert_info_free(tci);
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     /* get greeting */
     if ((e = smtp_get_greeting(&srv, msg, NULL, errstr)) != SMTP_EOK)
@@ -510,7 +508,7 @@ int msmtp_rmqs(account_t *acc, int debug, const char *rmqs_argument,
     }
     
     /* start tls for starttls servers */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls && !acc->tls_nostarttls)
     {
 	if (!(srv.cap.flags & SMTP_CAP_STARTTLS))
@@ -551,7 +549,7 @@ int msmtp_rmqs(account_t *acc, int debug, const char *rmqs_argument,
 	    return exitcode_smtp(e);
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     if (!(srv.cap.flags & SMTP_CAP_ETRN))
     {
@@ -610,9 +608,9 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
     char *server_address = NULL;
     char *server_greeting = NULL;
     int e;
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     tls_cert_info_t *tci = NULL;
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
     
     *errstr = NULL;
     *msg = NULL;
@@ -630,21 +628,22 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
     }
 
     /* prepare tls */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls)
     {
 	tci = tls_cert_info_new();
 	if ((e = smtp_tls_init(&srv, acc->tls_key_file, acc->tls_cert_file, 
-			acc->tls_trust_file, errstr)) != TLS_EOK)
+			acc->tls_trust_file, acc->tls_force_sslv3, errstr)) 
+		!= TLS_EOK)
 	{
 	    e = exitcode_tls(e);
 	    goto error_exit;
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     /* start tls for ssmtp servers */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls && acc->tls_nostarttls)
     {
 	if ((e = smtp_tls(&srv, acc->host, acc->tls_nocertcheck, tci, errstr))
@@ -655,7 +654,7 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
 	    goto error_exit;
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     /* get greeting */
     if ((e = smtp_get_greeting(&srv, msg, &server_greeting, 
@@ -675,7 +674,7 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
     }
     
     /* start tls for starttls servers */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls && !acc->tls_nostarttls)
     {
 	if (!(srv.cap.flags & SMTP_CAP_STARTTLS))
@@ -707,7 +706,7 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
 	    goto error_exit;
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     /* end session */
     msmtp_endsession(&srv, 1);
@@ -741,17 +740,17 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
     {
 	printf("    %s\n", msmtp_sanitize_string(server_greeting));
     }
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls)
     {
 	msmtp_print_tls_cert_info(tci);
     }
-#endif /* HAVE_SSL */
-#ifdef HAVE_SSL
+#endif /* HAVE_TLS */
+#ifdef HAVE_TLS
     if (srv.cap.flags == 0 && !(acc->tls && !acc->tls_nostarttls))
-#else /* not HAVE_SSL */
+#else /* not HAVE_TLS */
     if (srv.cap.flags == 0)
-#endif /* not HAVE_SSL */
+#endif /* not HAVE_TLS */
     {
 	printf(_("No special capabilities.\n"));
     }
@@ -796,12 +795,12 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
     	    printf("    DSN:\n        %s\n", _("Support for "
 			"Delivery Status Notifications"));
        	}
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 	if ((acc->tls && !acc->tls_nostarttls) 
 		|| (srv.cap.flags & SMTP_CAP_STARTTLS))
-#else /* not HAVE_SSL */
+#else /* not HAVE_TLS */
         if (srv.cap.flags & SMTP_CAP_STARTTLS)
-#endif /* not HAVE_SSL */
+#endif /* not HAVE_TLS */
 	{
     	    printf("    STARTTLS:\n        %s\n", _("Support for "
 			"TLS encryption via the STARTTLS command"));
@@ -840,11 +839,11 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
     	    }
 	    printf("\n");
        	}
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 	if ((srv.cap.flags & SMTP_CAP_STARTTLS) && !acc->tls)
-#else /* not HAVE_SSL */
+#else /* not HAVE_TLS */
 	if (srv.cap.flags & SMTP_CAP_STARTTLS)
-#endif /* not HAVE_SSL */
+#endif /* not HAVE_TLS */
 	{
     	    printf(_("This server might advertise more or other "
 		    "capabilities when TLS is active.\n"));
@@ -856,12 +855,12 @@ int msmtp_serverinfo(account_t *acc, int debug, list_t **msg, char **errstr)
 error_exit:
     free(server_canonical_name);
     free(server_address);
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (tci)
     {
 	tls_cert_info_free(tci);
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
     free(server_greeting);
     return e;
 }
@@ -1338,9 +1337,9 @@ int msmtp_sendmail(account_t *acc, list_t *recipients, int read_recipients,
     smtp_server_t srv;
     FILE *tmpfile = NULL;
     int e;
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     tls_cert_info_t *tci = NULL;
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
     
     *errstr = NULL;
     *msg = NULL;
@@ -1367,17 +1366,18 @@ int msmtp_sendmail(account_t *acc, list_t *recipients, int read_recipients,
     srv = smtp_new(debug ? stdout : NULL, acc->protocol);
 
     /* prepare tls */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls)
     {
 	if ((e = smtp_tls_init(&srv, acc->tls_key_file, acc->tls_cert_file, 
-			acc->tls_trust_file, errstr)) != TLS_EOK)
+			acc->tls_trust_file, acc->tls_force_sslv3, errstr)) 
+		!= TLS_EOK)
 	{
 	    e = exitcode_tls(e);
 	    goto error_exit;
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     /* connect */
     if ((e = smtp_connect(&srv, acc->host, acc->port, acc->timeout,
@@ -1388,7 +1388,7 @@ int msmtp_sendmail(account_t *acc, list_t *recipients, int read_recipients,
     }
 
     /* start tls for ssmtp servers */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls && acc->tls_nostarttls)
     {
 	if (debug)
@@ -1412,7 +1412,7 @@ int msmtp_sendmail(account_t *acc, list_t *recipients, int read_recipients,
 	    tls_cert_info_free(tci);
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     /* get greeting */
     if ((e = smtp_get_greeting(&srv, msg, NULL, errstr)) != SMTP_EOK)
@@ -1431,7 +1431,7 @@ int msmtp_sendmail(account_t *acc, list_t *recipients, int read_recipients,
     }
 
     /* start tls for starttls servers */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (acc->tls && !acc->tls_nostarttls)
     {
 	if (!(srv.cap.flags & SMTP_CAP_STARTTLS))
@@ -1476,7 +1476,7 @@ int msmtp_sendmail(account_t *acc, list_t *recipients, int read_recipients,
 	    goto error_exit;
 	}
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
 
     /* test for needed features */
     if ((acc->dsn_return || acc->dsn_notify) && !(srv.cap.flags & SMTP_CAP_DSN))
@@ -1837,7 +1837,7 @@ void msmtp_log_to_file(const char *logfile, const char *loginfo)
 	    failure_reason = xasprintf(_("cannot open: %s"), strerror(errno));
 	    goto log_failure;
 	}
-	if ((e = lock_file(f, OSENV_LOCK_WRITE, 10)) != 0)
+	if ((e = lock_file(f, TOOLS_LOCK_WRITE, 10)) != 0)
 	{
 	    if (e == 1)
 	    {
@@ -1983,7 +1983,7 @@ void msmtp_print_version(void)
     printf(_("%s version %s\n"), PACKAGE_NAME, VERSION);
     /* TLS/SSL support */
     printf(_("TLS/SSL library: %s\n"),
-#ifdef HAVE_GNUTLS
+#ifdef HAVE_LIBGNUTLS
 	    "GnuTLS"
 #elif defined (HAVE_OPENSSL)
     	    "OpenSSL"
@@ -1994,11 +1994,11 @@ void msmtp_print_version(void)
     /* Authentication support */
     printf(_("Authentication library: %s\n"
 		"Supported authentication methods:\n"),
-#ifdef USE_GSASL
+#ifdef HAVE_LIBGSASL
 	    "GNU SASL"
 #else
 	    _("built-in")
-#endif /* USE_GSASL */
+#endif /* HAVE_LIBGSASL */
 	  );
     if (smtp_client_supports_authmech("PLAIN"))
     {
@@ -2031,7 +2031,7 @@ void msmtp_print_version(void)
     printf("\n");
     /* Internationalized Domain Names support */
     printf(_("IDN support: "));
-#ifdef USE_LIBIDN
+#ifdef HAVE_LIBIDN
     printf(_("enabled"));
 #else
     printf(_("disabled"));
@@ -2057,11 +2057,12 @@ void msmtp_print_version(void)
     free(userconffile);
     free(homedir);
     printf("\n");
-    printf(_("Copyright (C) 2006 Martin Lambers and others.\n"
-    		"This is free software; see the source for copying "
-		"conditions.  There is NO\n"
-		"warranty; not even for MERCHANTABILITY or FITNESS FOR A "
-		"PARTICULAR PURPOSE.\n"));
+    printf(_("Copyright (C) 2007 Martin Lambers and others.\n"
+		"This is free software.  You may redistribute copies of "
+		    "it under the terms of\n"
+		"the GNU General Public License "
+		    "<http://www.gnu.org/licenses/gpl.html>.\n"
+		"There is NO WARRANTY, to the extent permitted by law.\n"));
 }
 
 
@@ -2120,6 +2121,7 @@ void msmtp_print_help(void)
 	    "  --user=[username]            Set/unset user name for "
 	    	"authentication.\n"
 	    "  --tls[=(on|off)]             Enable/disable TLS encryption.\n"
+	    "  --tls-starttls[=(on|off)]    Enable/disable STARTTLS for TLS.\n"
 	    "  --tls-trust-file=[file]      Set/unset trust file for TLS.\n"
             "  --tls-key-file=[file]        Set/unset private key file for "
 	    	"TLS.\n"
@@ -2127,7 +2129,8 @@ void msmtp_print_help(void)
 	    	"TLS.\n"
 	    "  --tls-certcheck[=(on|off)]   Enable/disable server certificate "
 	    	"checks for TLS.\n"
-	    "  --tls-starttls[=(on|off)]    Enable/disable STARTTLS for TLS.\n"
+	    "  --tls-force-sslv3[=(on|off)] Enable/disable restriction to "
+	        "SSLv3.\n"
             "Options specific to sendmail mode:\n"
 	    "  --auto-from[=(on|off)]       Enable/disable automatic "
 	        "envelope-from addresses.\n"
@@ -2189,18 +2192,19 @@ typedef struct
 #define LONGONLYOPT_AUTH		5
 #define LONGONLYOPT_USER		6
 #define LONGONLYOPT_TLS			7
-#define LONGONLYOPT_TLS_TRUST_FILE	8
-#define LONGONLYOPT_TLS_KEY_FILE	9
-#define LONGONLYOPT_TLS_CERT_FILE	10
-#define LONGONLYOPT_TLS_CERTCHECK	11
-#define LONGONLYOPT_TLS_STARTTLS	12
-#define LONGONLYOPT_PROTOCOL		13
-#define LONGONLYOPT_DOMAIN		14
-#define LONGONLYOPT_KEEPBCC		15
-#define LONGONLYOPT_RMQS		16
-#define LONGONLYOPT_SYSLOG		17
-#define LONGONLYOPT_MAILDOMAIN		18
-#define LONGONLYOPT_AUTO_FROM		19
+#define LONGONLYOPT_TLS_STARTTLS	8
+#define LONGONLYOPT_TLS_TRUST_FILE	9
+#define LONGONLYOPT_TLS_KEY_FILE	10
+#define LONGONLYOPT_TLS_CERT_FILE	11
+#define LONGONLYOPT_TLS_CERTCHECK	12
+#define LONGONLYOPT_TLS_FORCE_SSLV3	13
+#define LONGONLYOPT_PROTOCOL		14
+#define LONGONLYOPT_DOMAIN		15
+#define LONGONLYOPT_KEEPBCC		16
+#define LONGONLYOPT_RMQS		17
+#define LONGONLYOPT_SYSLOG		18
+#define LONGONLYOPT_MAILDOMAIN		19
+#define LONGONLYOPT_AUTO_FROM		20
 
 int msmtp_cmdline(msmtp_cmdline_conf_t *conf, int argc, char *argv[])
 {
@@ -2226,11 +2230,13 @@ int msmtp_cmdline(msmtp_cmdline_conf_t *conf, int argc, char *argv[])
 	{ "auth",            optional_argument, 0, LONGONLYOPT_AUTH },
 	{ "user",            required_argument, 0, LONGONLYOPT_USER },
 	{ "tls",             optional_argument, 0, LONGONLYOPT_TLS },
+	{ "tls-starttls",    optional_argument, 0, LONGONLYOPT_TLS_STARTTLS },
 	{ "tls-trust-file",  required_argument, 0, LONGONLYOPT_TLS_TRUST_FILE },
 	{ "tls-key-file",    required_argument, 0, LONGONLYOPT_TLS_KEY_FILE },
 	{ "tls-cert-file",   required_argument, 0, LONGONLYOPT_TLS_CERT_FILE },
 	{ "tls-certcheck",   optional_argument, 0, LONGONLYOPT_TLS_CERTCHECK },
-	{ "tls-starttls",    optional_argument, 0, LONGONLYOPT_TLS_STARTTLS },
+	{ "tls-force-sslv3", optional_argument, 0, 
+	    LONGONLYOPT_TLS_FORCE_SSLV3 },
 	{ "dsn-notify",      required_argument, 0, 'N' },
 	{ "dsn-return",      required_argument, 0, 'R' },
 	{ "protocol",        required_argument, 0, LONGONLYOPT_PROTOCOL },
@@ -2480,6 +2486,24 @@ int msmtp_cmdline(msmtp_cmdline_conf_t *conf, int argc, char *argv[])
 		}
 		conf->cmdline_account->mask |= ACC_TLS;
 		break;
+		
+	    case LONGONLYOPT_TLS_STARTTLS:
+	    	if (!optarg || is_on(optarg))
+    		{
+		    conf->cmdline_account->tls_nostarttls = 0;
+		}
+		else if (is_off(optarg))
+		{
+		    conf->cmdline_account->tls_nostarttls = 1;
+		}
+		else
+		{
+		    print_error(_("invalid argument %s for %s"), 
+			    optarg, "--tls-starttls");
+		    error_code = 1;
+		}
+		conf->cmdline_account->mask |= ACC_TLS_NOSTARTTLS;
+		break;
 
 	    case LONGONLYOPT_TLS_TRUST_FILE:
 		free(conf->cmdline_account->tls_trust_file);
@@ -2539,22 +2563,22 @@ int msmtp_cmdline(msmtp_cmdline_conf_t *conf, int argc, char *argv[])
 		conf->cmdline_account->mask |= ACC_TLS_NOCERTCHECK;
 		break;
 
-	    case LONGONLYOPT_TLS_STARTTLS:
-	    	if (!optarg || is_on(optarg))
+	    case LONGONLYOPT_TLS_FORCE_SSLV3:
+	   	if (!optarg || is_on(optarg))
     		{
-		    conf->cmdline_account->tls_nostarttls = 0;
+		    conf->cmdline_account->tls_force_sslv3 = 1;
 		}
 		else if (is_off(optarg))
 		{
-		    conf->cmdline_account->tls_nostarttls = 1;
+		    conf->cmdline_account->tls_force_sslv3 = 0;
 		}
 		else
 		{
 		    print_error(_("invalid argument %s for %s"), 
-			    optarg, "--tls-starttls");
+			    optarg, "--tls-force-sslv3");
 		    error_code = 1;
 		}
-		conf->cmdline_account->mask |= ACC_TLS_NOSTARTTLS;
+		conf->cmdline_account->mask |= ACC_TLS_FORCE_SSLV3;
 		break;
 
 	    case 'N':
@@ -2942,20 +2966,22 @@ void msmtp_print_conf(msmtp_cmdline_conf_t conf, account_t *account)
 	    "password        = %s\n"
 	    "ntlmdomain      = %s\n"
 	    "tls             = %s\n"
+	    "tls_starttls    = %s\n"
 	    "tls_trust_file  = %s\n"
 	    "tls_key_file    = %s\n"
 	    "tls_cert_file   = %s\n"
-	    "tls_starttls    = %s\n"
-	    "tls_certcheck   = %s\n",
+	    "tls_certcheck   = %s\n"
+	    "tls_force_sslv3 = %s\n",
 	    account->username ? account->username : _("(not set)"),
 	    account->password ? "*" : _("(not set)"),
 	    account->ntlmdomain ? account->ntlmdomain : _("(not set)"),
 	    account->tls ? _("on") : _("off"), 
+	    account->tls_nostarttls ? _("off") : _("on"),
 	    account->tls_trust_file ? account->tls_trust_file : _("(not set)"),
 	    account->tls_key_file ? account->tls_key_file : _("(not set)"),
 	    account->tls_cert_file ? account->tls_cert_file : _("(not set)"),
-	    account->tls_nostarttls ? _("off") : _("on"),
-	    account->tls_nocertcheck ? _("off") : _("on"));
+	    account->tls_nocertcheck ? _("off") : _("on"),
+	    account->tls_force_sslv3 ? _("on") : _("off"));
     if (conf.sendmail)
     {
 	printf("auto_from       = %s\n"
@@ -3012,7 +3038,7 @@ int main(int argc, char *argv[])
     int e;
     list_t *lp;
     /* misc */
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     int tls_lib_initialized = 0;
 #endif
     int net_lib_initialized = 0;
@@ -3032,8 +3058,10 @@ int main(int argc, char *argv[])
     /* Avoid the side effects of text mode interpretations on DOS systems. */
 #ifdef W32_NATIVE
     _setmode(_fileno(stdin), _O_BINARY);
+    _fmode = _O_BINARY;
 #elif defined DJGPP
     setmode(fileno(stdin), O_BINARY);
+    _fmode = O_BINARY;
 #endif
 
     errstr = NULL;
@@ -3233,7 +3261,7 @@ int main(int argc, char *argv[])
     net_lib_initialized = 1;
     if (account->tls)
     {
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
 	if ((e = tls_lib_init(&errstr)) != TLS_EOK)
 	{
 	    print_error(_("cannot initialize TLS library: %s"), 
@@ -3242,11 +3270,11 @@ int main(int argc, char *argv[])
 	    goto exit;
 	}
 	tls_lib_initialized = 1;
-#else /* not HAVE_SSL */
+#else /* not HAVE_TLS */
 	print_error(_("support for TLS is not compiled in"));
 	error_code = EX_UNAVAILABLE;
 	goto exit;
-#endif /* not HAVE_SSL */
+#endif /* not HAVE_TLS */
     }
 
     /* do the work */
@@ -3395,12 +3423,12 @@ exit:
     /* clean up */
     free(loaded_system_conffile);
     free(loaded_user_conffile);
-#ifdef HAVE_SSL
+#ifdef HAVE_TLS
     if (tls_lib_initialized) 
     {
 	tls_lib_deinit();
     }
-#endif /* HAVE_SSL */
+#endif /* HAVE_TLS */
     if (net_lib_initialized)
     {
 	net_lib_deinit();
